@@ -31,14 +31,15 @@
 
 """
 
+import errno
+import email
 import logging
-import mimetools
-import OpenSSL
 import select
 import socket
-import errno
 
-from cStringIO import StringIO
+from io import StringIO
+
+import OpenSSL
 
 from ganeti import constants
 from ganeti import utils
@@ -88,6 +89,12 @@ _SSL_UNEXPECTED_EOF = "Unexpected EOF"
 # send/receive quantum
 SOCK_BUF_SIZE = 32768
 
+# OpenSSL.SSL.ConnectionType was deprecated in pyopenssl-19.1.0:
+try:
+    SSL_CONN_TYPE = OpenSSL.SSL.Connection
+except AttributeError:
+    SSL_CONN_TYPE = OpenSSL.SSL.ConnectionType
+
 
 class HttpError(Exception):
   """Internal exception for HTTP errors.
@@ -335,10 +342,10 @@ class HttpVersionNotSupported(HttpException):
 def ParseHeaders(buf):
   """Parses HTTP headers.
 
-  @note: This is just a trivial wrapper around C{mimetools.Message}
+  @note: This is just a trivial wrapper around C{email.message_from_file}
 
   """
-  return mimetools.Message(buf, 0)
+  return email.message_from_file(buf)
 
 
 def SocketOperation(sock, op, arg1, timeout):
@@ -377,7 +384,7 @@ def SocketOperation(sock, op, arg1, timeout):
 
   # Handshake is only supported by SSL sockets
   if (op == SOCKOP_HANDSHAKE and
-      not isinstance(sock, OpenSSL.SSL.ConnectionType)):
+      not isinstance(sock, SSL_CONN_TYPE)):
     return
 
   # No override by default
@@ -408,13 +415,20 @@ def SocketOperation(sock, op, arg1, timeout):
     try:
       try:
         if op == SOCKOP_SEND:
+          # Non-SSL sockets expect bytes
+          if isinstance(sock, socket.socket):
+            data = arg1.encode("utf-8")
+            # Use sendall to avoid partial writes that could cause desync with
+            # our caller, as len(data) != len(arg1) in the general case
+            sock.sendall(data)
+            return len(data)
           return sock.send(arg1)
 
         elif op == SOCKOP_RECV:
           return sock.recv(arg1)
 
         elif op == SOCKOP_SHUTDOWN:
-          if isinstance(sock, OpenSSL.SSL.ConnectionType):
+          if isinstance(sock, SSL_CONN_TYPE):
             # PyOpenSSL's shutdown() doesn't take arguments
             return sock.shutdown()
           else:
@@ -436,7 +450,7 @@ def SocketOperation(sock, op, arg1, timeout):
       except OpenSSL.SSL.WantX509LookupError:
         continue
 
-      except OpenSSL.SSL.ZeroReturnError, err:
+      except OpenSSL.SSL.ZeroReturnError as err:
         # SSL Connection has been closed. In SSL 3.0 and TLS 1.0, this only
         # occurs if a closure alert has occurred in the protocol, i.e. the
         # connection has been closed cleanly. Note that this does not
@@ -451,7 +465,7 @@ def SocketOperation(sock, op, arg1, timeout):
         # SSL_shutdown shouldn't return SSL_ERROR_ZERO_RETURN
         raise socket.error(err.args)
 
-      except OpenSSL.SSL.SysCallError, err:
+      except OpenSSL.SSL.SysCallError as err:
         if op == SOCKOP_SEND:
           # arg1 is the data when writing
           if err.args and err.args[0] == -1 and arg1 == "":
@@ -469,10 +483,10 @@ def SocketOperation(sock, op, arg1, timeout):
 
         raise socket.error(err.args)
 
-      except OpenSSL.SSL.Error, err:
+      except OpenSSL.SSL.Error as err:
         raise socket.error(err.args)
 
-    except socket.error, err:
+    except socket.error as err:
       if err.args and err.args[0] == errno.EAGAIN:
         # Ignore EAGAIN
         continue
@@ -498,7 +512,7 @@ def ShutdownConnection(sock, close_timeout, write_timeout, msgreader, force):
       waiting for peer
 
   """
-  #print msgreader.peer_will_close, force
+  #print(msgreader.peer_will_close, force)
   if msgreader and msgreader.peer_will_close and not force:
     # Wait for peer to close
     try:
@@ -516,7 +530,7 @@ def ShutdownConnection(sock, close_timeout, write_timeout, msgreader, force):
                     write_timeout)
   except HttpSocketTimeout:
     raise HttpError("Timeout while shutting down connection")
-  except socket.error, err:
+  except socket.error as err:
     # Ignore ENOTCONN
     if not (err.args and err.args[0] == errno.ENOTCONN):
       raise HttpError("Error while shutting down connection: %s" % err)
@@ -535,7 +549,7 @@ def Handshake(sock, write_timeout):
     return SocketOperation(sock, SOCKOP_HANDSHAKE, None, write_timeout)
   except HttpSocketTimeout:
     raise HttpError("Timeout during SSL handshake")
-  except socket.error, err:
+  except socket.error as err:
     raise HttpError("Error in SSL handshake: %s" % err)
 
 
@@ -754,14 +768,14 @@ class HttpMessageWriter(object):
 
     # Add headers
     if self._msg.start_line.version != HTTP_0_9:
-      for name, value in self._msg.headers.iteritems():
+      for name, value in self._msg.headers.items():
         buf.write("%s: %s\r\n" % (name, value))
 
     buf.write("\r\n")
 
     # Add message body if needed
     if self.HasMessageBody():
-      buf.write(self._msg.body)
+      buf.write(self._msg.body.decode())
 
     elif self._msg.body:
       logging.warning("Ignoring message body")
@@ -820,7 +834,7 @@ class HttpMessageReader(object):
       data = SocketOperation(sock, SOCKOP_RECV, SOCK_BUF_SIZE, read_timeout)
 
       if data:
-        buf += data
+        buf += data.decode()
       else:
         eof = True
 
