@@ -37,17 +37,14 @@ import os
 import stat
 import errno
 import socket
-import StringIO
+import io
 import logging
-try:
-  import fdsend   # pylint: disable=F0401
-except ImportError:
-  fdsend = None
 
 from bitarray import bitarray
 
 from ganeti import errors
 from ganeti import utils
+from ganeti import constants
 from ganeti import serializer
 
 
@@ -104,7 +101,7 @@ class QmpMessage(object):
     """Delete the specified element from the QmpMessage.
 
     """
-    del(self.data[key])
+    del self.data[key]
 
   @staticmethod
   def BuildFromJsonString(json_string):
@@ -120,7 +117,7 @@ class QmpMessage(object):
     data = serializer.LoadJson(json_string)
     return QmpMessage(data)
 
-  def __str__(self):
+  def to_bytes(self):
     # The protocol expects the JSON object to be sent as a single line.
     return serializer.DumpJson(self.data)
 
@@ -148,7 +145,7 @@ class MonitorSocket(object):
     sock_stat = None
     try:
       sock_stat = os.stat(self.monitor_filename)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       if err.errno == errno.ENOENT:
         raise errors.HypervisorError("No monitor socket found")
       else:
@@ -254,12 +251,16 @@ class QmpConnection(MonitorSocket):
   _QEMU_KEY = "qemu"
   _CAPABILITIES_COMMAND = "qmp_capabilities"
   _QUERY_COMMANDS = "query-commands"
-  _MESSAGE_END_TOKEN = "\r\n"
-  _QEMU_PCI_SLOTS = 32 # The number of PCI slots QEMU exposes by default
+  _MESSAGE_END_TOKEN = b"\r\n"
+  # List of valid attributes for the device_add QMP command.
+  # Extra attributes found in device's hvinfo will be ignored.
+  _DEVICE_ATTRIBUTES = [
+    "driver", "id", "bus", "addr", "channel", "scsi-id", "lun"
+    ]
 
   def __init__(self, monitor_filename):
     super(QmpConnection, self).__init__(monitor_filename)
-    self._buf = ""
+    self._buf = b""
     self.supported_commands = None
 
   def __enter__(self):
@@ -299,7 +300,7 @@ class QmpConnection(MonitorSocket):
 
     # This is needed because QMP can return more than one greetings
     # see https://groups.google.com/d/msg/ganeti-devel/gZYcvHKDooU/SnukC8dgS5AJ
-    self._buf = ""
+    self._buf = b""
 
     # Let's put the monitor in command mode using the qmp_capabilities
     # command, or else no command will be executable.
@@ -324,7 +325,7 @@ class QmpConnection(MonitorSocket):
     if pos >= 0:
       try:
         message = QmpMessage.BuildFromJsonString(buf[:pos + 1])
-      except Exception, err:
+      except Exception as err:
         raise errors.ProgrammerError("QMP data serialization error: %s" % err)
       buf = buf[pos + 1:]
 
@@ -346,7 +347,7 @@ class QmpConnection(MonitorSocket):
     if message:
       return message
 
-    recv_buffer = StringIO.StringIO(self._buf)
+    recv_buffer = io.BytesIO(self._buf)
     recv_buffer.seek(len(self._buf))
     try:
       while True:
@@ -359,10 +360,10 @@ class QmpConnection(MonitorSocket):
         if message:
           return message
 
-    except socket.timeout, err:
+    except socket.timeout as err:
       raise errors.HypervisorError("Timeout while receiving a QMP message: "
                                    "%s" % (err))
-    except socket.error, err:
+    except socket.error as err:
       raise errors.HypervisorError("Unable to receive data from KVM using the"
                                    " QMP protocol: %s" % err)
 
@@ -377,16 +378,11 @@ class QmpConnection(MonitorSocket):
     """
     self._check_connection()
     try:
-      message_str = str(message)
-    except Exception, err:
-      raise errors.ProgrammerError("QMP data deserialization error: %s" % err)
-
-    try:
-      self.sock.sendall(message_str)
-    except socket.timeout, err:
+      self.sock.sendall(message.to_bytes())
+    except socket.timeout as err:
       raise errors.HypervisorError("Timeout while sending a QMP message: "
-                                   "%s (%s)" % (err.string, err.errno))
-    except socket.error, err:
+                                   "%s" % err)
+    except socket.error as err:
       raise errors.HypervisorError("Unable to send data from KVM using the"
                                    " QMP protocol: %s" % err)
 
@@ -459,6 +455,15 @@ class QmpConnection(MonitorSocket):
 
       return response[self._RETURN_KEY]
 
+  def _filter_hvinfo(self, hvinfo):
+    """Filter non valid keys of the device's hvinfo (if any)."""
+    ret = {}
+    for k in self._DEVICE_ATTRIBUTES:
+      if k in hvinfo:
+        ret[k] = hvinfo[k]
+
+    return ret
+
   @_ensure_connection
   def HotAddNic(self, nic, devid, tapfds=None, vhostfds=None, features=None):
     """Hot-add a NIC
@@ -501,13 +506,12 @@ class QmpConnection(MonitorSocket):
     self.Execute("netdev_add", arguments)
 
     arguments = {
-      "driver": "virtio-net-pci",
-      "id": devid,
-      "bus": "pci.0",
-      "addr": hex(nic.pci),
       "netdev": devid,
       "mac": nic.mac,
     }
+    # Note that hvinfo that _GenerateDeviceHVInfo() creates
+    # sould include *only* the driver, id, bus, and addr keys
+    arguments.update(self._filter_hvinfo(nic.hvinfo))
     if enable_mq:
       arguments.update({
         "mq": "on",
@@ -524,12 +528,13 @@ class QmpConnection(MonitorSocket):
     self.Execute("netdev_del", {"id": devid})
 
   @_ensure_connection
-  def HotAddDisk(self, disk, devid, uri):
+  def HotAddDisk(self, disk, devid, uri, drive_add_fn=None):
     """Hot-add a disk
 
     Try opening the device to obtain a fd and pass it with SCM_RIGHTS. This
     will be omitted in case of userspace access mode (open will fail).
-    Then use blockdev-add and then device_add.
+    Then use blockdev-add QMP command or drive_add_fn() callback if any.
+    The add the guest device.
 
     """
     if os.path.exists(uri):
@@ -543,28 +548,41 @@ class QmpConnection(MonitorSocket):
       filename = uri
       fdset = None
 
-    arguments = {
-      "options": {
-        "driver": "raw",
-        "id": devid,
-        "file": {
-          "driver": "file",
-          "filename": filename,
+    # FIXME: Use blockdev-add/blockdev-del when properly implemented in QEMU.
+    # This is an ugly hack to work around QEMU commits 48f364dd and da2cf4e8:
+    #  * HMP's drive_del is not supported any more on a drive added
+    #    via QMP's blockdev-add
+    #  * Stay away from immature blockdev-add unless you want to help
+    #     with development.
+    # Using drive_add here must be done via a callback due to the fact that if
+    # a QMP connection terminates before a drive keeps a reference to the fd
+    # passed via the add-fd QMP command, then the fd gets closed and
+    # cannot be used later.
+    if drive_add_fn:
+      drive_add_fn(filename)
+    else:
+      arguments = {
+        "options": {
+          "driver": "raw",
+          "id": devid,
+          "file": {
+            "driver": "file",
+            "filename": filename,
+          }
         }
       }
-    }
-    self.Execute("blockdev-add", arguments)
+      self.Execute("blockdev-add", arguments)
 
     if fdset is not None:
       self._RemoveFdset(fdset)
 
     arguments = {
-      "driver": "virtio-blk-pci",
-      "id": devid,
-      "bus": "pci.0",
-      "addr": hex(disk.pci),
       "drive": devid,
     }
+    # Note that hvinfo that _GenerateDeviceHVInfo() creates
+    # sould include *only* the driver, id, bus, and
+    # addr or channel, scsi-id, and lun keys
+    arguments.update(self._filter_hvinfo(disk.hvinfo))
     self.Execute("device_add", arguments)
 
   @_ensure_connection
@@ -589,26 +607,60 @@ class QmpConnection(MonitorSocket):
     devices = bus["devices"]
     return devices
 
-  @_ensure_connection
-  def HasPCIDevice(self, device, devid):
-    """Check if a specific device exists or not on a running instance.
-
-    It will match the PCI slot of the device and the id currently
-    obtained by _GenerateDeviceKVMId().
+  def _HasPCIDevice(self, devid):
+    """Check if a specific device ID exists on the PCI bus.
 
     """
     for d in self._GetPCIDevices():
-      if d["qdev_id"] == devid and d["slot"] == device.pci:
+      if d["qdev_id"] == devid:
         return True
 
     return False
 
+  def _GetBlockDevices(self):
+    """Get the block devices of a running instance.
+
+    The query-block QMP command returns a list of dictionaries
+    including information for each virtual disk. For example:
+
+    [{"device": "disk-049f140d", "inserted": {"file": ..., "image": ...}}]
+
+    @rtype: list of dicts
+    @return: Info about the virtual disks of the instance.
+
+    """
+    self._check_connection()
+    devices = self.Execute("query-block")
+    return devices
+
+  def _HasBlockDevice(self, devid):
+    """Check if a specific device ID exists among block devices.
+
+    """
+    for d in self._GetBlockDevices():
+      if d["device"] == devid:
+        return True
+
+    return False
+
+  @_ensure_connection
+  def HasDevice(self, devid):
+    """Check if a specific device exists or not on a running instance.
+
+    It first checks the PCI devices and then the block devices.
+
+    """
+    if (self._HasPCIDevice(devid) or self._HasBlockDevice(devid)):
+      return True
+
+    return False
+
   @_ensure_connection
   def GetFreePCISlot(self):
     """Get the first available PCI slot of a running instance.
 
     """
-    slots = bitarray(self._QEMU_PCI_SLOTS)
+    slots = bitarray(constants.QEMU_PCI_SLOTS)
     slots.setall(False) # pylint: disable=E1101
     for d in self._GetPCIDevices():
       slot = d["slot"]
@@ -620,17 +672,13 @@ class QmpConnection(MonitorSocket):
   def CheckDiskHotAddSupport(self):
     """Check if disk hotplug is possible
 
-    Hotplug is *not* supported in case:
-     - fdsend module is missing
-     - add-fd and blockdev-add qmp commands are not supported
+    Hotplug is *not* supported in case the add-fd and blockdev-add qmp commands
+    are not supported
 
     """
     def _raise(reason):
       raise errors.HotplugError("Cannot hot-add disk: %s." % reason)
 
-    if not fdsend:
-      _raise("fdsend python module is missing")
-
     if "add-fd" not in self.supported_commands:
       _raise("add-fd qmp command is not supported")
 
@@ -641,29 +689,114 @@ class QmpConnection(MonitorSocket):
   def CheckNicHotAddSupport(self):
     """Check if NIC hotplug is possible
 
-    Hotplug is *not* supported in case:
-     - fdsend module is missing
-     - getfd and netdev_add qmp commands are not supported
+    Hotplug is *not* supported in case the getfd and netdev_add qmp commands
+    are not supported
 
     """
     def _raise(reason):
       raise errors.HotplugError("Cannot hot-add NIC: %s." % reason)
 
-    if not fdsend:
-      _raise("fdsend python module is missing")
-
     if "getfd" not in self.supported_commands:
       _raise("getfd qmp command is not supported")
 
     if "netdev_add" not in self.supported_commands:
       _raise("netdev_add qmp command is not supported")
 
+  @_ensure_connection
+  def SetMigrationParameters(self, max_bandwidth, downtime_limit):
+    """Configute live migration parameters
+
+    """
+
+    arguments = {
+      "max-bandwidth": max_bandwidth,
+      "downtime-limit": downtime_limit,
+    }
+
+    if self.version >= (3, 0, 0):
+      arguments["max-postcopy-bandwidth"] = max_bandwidth
+
+    self.Execute("migrate-set-parameters", arguments)
+
+  @_ensure_connection
+  def SetMigrationCapabilities(self, capabilities, state):
+    """Configure live migration capabilities
+
+    """
+
+    arguments = {
+      "capabilities": []
+    }
+    for capability in capabilities:
+      arguments["capabilities"].append({
+        "capability": capability,
+        "state": state,
+      })
+
+    self.Execute("migrate-set-capabilities", arguments)
+
+  @_ensure_connection
+  def StopGuestEmulation(self):
+    """Pause the running guest
+
+    """
+
+    self.Execute("stop")
+
+  @_ensure_connection
+  def ContinueGuestEmulation(self):
+    """Continue the previously paused guest
+
+    """
+
+    self.Execute("cont")
+
+  @_ensure_connection
+  def StartMigration(self, target, port):
+    """Start migration of an instance
+
+    """
+
+    arguments = {
+      "uri": "tcp:%s:%s" % (target, port)
+    }
+
+    self.Execute("migrate", arguments)
+
+  @_ensure_connection
+  def StartPostcopyMigration(self):
+    """ Start postcopy-ram migration
+
+    """
+
+    self.Execute("migrate-start-postcopy")
+
+  @_ensure_connection
+  def GetMigrationStatus(self):
+    """Retrieve the current migration status
+
+    """
+
+    return self.Execute("query-migrate")
+
+  @_ensure_connection
+  def SetSpicePassword(self, spice_pwd):
+    """Set Spice password of an instance
+
+    """
+    arguments = {
+      "protocol": "spice",
+      "password": spice_pwd,
+    }
+
+    self.Execute("set_password", arguments)
+
   def _GetFd(self, fd, fdname):
     """Wrapper around the getfd qmp command
 
-    Use fdsend to send an fd to a running process via SCM_RIGHTS and then use
-    the getfd qmp command to name it properly so that it can be used
-    later by NIC hotplugging.
+    Send an fd to a running process via SCM_RIGHTS and then use the getfd qmp
+    command to name it properly so that it can be used later by NIC
+    hotplugging.
 
     @type fd: int
     @param fd: The file descriptor to pass
@@ -672,21 +805,20 @@ class QmpConnection(MonitorSocket):
     """
     self._check_connection()
     try:
-      fdsend.sendfds(self.sock, " ", fds=[fd])
+      utils.SendFds(self.sock, b" ", [fd])
       arguments = {
           "fdname": fdname,
           }
       self.Execute("getfd", arguments)
-    except errors.HypervisorError, err:
+    except errors.HypervisorError as err:
       logging.info("Passing fd %s via SCM_RIGHTS failed: %s", fd, err)
       raise
 
   def _AddFd(self, fd):
     """Wrapper around add-fd qmp command
 
-    Use fdsend to send fd to a running process via SCM_RIGHTS and then add-fd
-    qmp command to add it to an fdset so that it can be used later by
-    disk hotplugging.
+    Send fd to a running process via SCM_RIGHTS and then add-fd qmp command to
+    add it to an fdset so that it can be used later by disk hotplugging.
 
     @type fd: int
     @param fd: The file descriptor to pass
@@ -697,11 +829,11 @@ class QmpConnection(MonitorSocket):
     """
     self._check_connection()
     try:
-      fdsend.sendfds(self.sock, " ", fds=[fd])
+      utils.SendFds(self.sock, b" ", [fd])
       # Omit fdset-id and let qemu create a new one (see qmp-commands.hx)
       response = self.Execute("add-fd")
       fdset = response["fdset-id"]
-    except errors.HypervisorError, err:
+    except errors.HypervisorError as err:
       logging.info("Passing fd %s via SCM_RIGHTS failed: %s", fd, err)
       raise
 
@@ -718,7 +850,7 @@ class QmpConnection(MonitorSocket):
     # Omit the fd to cleanup all fds in the fdset (see qemu/qmp-commands.hx)
     try:
       self.Execute("remove-fd", {"fdset-id": fdset})
-    except errors.HypervisorError, err:
+    except errors.HypervisorError as err:
       # There is no big deal if we cannot remove an fdset. This cleanup here is
       # done on a best effort basis. Upon next hot-add a new fdset will be
       # created. If we raise an exception here, that is after drive_add has
