@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2010, 2011 Google Inc.
@@ -132,10 +132,10 @@ class QmpStub(threading.Thread):
         break
       response = self.script.pop(0)
       if isinstance(response, str):
-        conn.send(response)
+        conn.send(response.encode("utf-8"))
       elif isinstance(response, list):
         for chunk in response:
-          conn.send(chunk)
+          conn.send(chunk.encode("utf-8"))
       else:
         raise errors.ProgrammerError("Unknown response type for %s" % response)
 
@@ -145,6 +145,9 @@ class QmpStub(threading.Thread):
     return (serializer.DumpJson(message) +
             hv_kvm.QmpConnection._MESSAGE_END_TOKEN)
 
+  def shutdown(self):
+    self.socket.close()
+
 
 class TestQmpMessage(testutils.GanetiTestCase):
   def testSerialization(self):
@@ -157,7 +160,7 @@ class TestQmpMessage(testutils.GanetiTestCase):
     for k, v in test_data.items():
       self.assertEqual(message[k], v)
 
-    serialized = str(message)
+    serialized = message.to_bytes()
     self.assertEqual(len(serialized.splitlines()), 1,
                      msg="Got multi-line message")
 
@@ -215,21 +218,26 @@ class TestQmp(testutils.GanetiTestCase):
 
     # Set up the QMP connection
     qmp_connection = hv_kvm.QmpConnection(socket_file.name)
-    qmp_connection.connect()
 
-    # Format the script
-    for request, expected_response in zip(self.REQUESTS,
-                                          self.EXPECTED_RESPONSES):
-      response = qmp_connection.Execute(request["execute"],
-                                        request["arguments"])
-      self.assertEqual(response, expected_response)
-      msg = hv_kvm.QmpMessage({"return": expected_response})
-      self.assertEqual(len(str(msg).splitlines()), 1,
-                       msg="Got multi-line message")
+    try:
+      qmp_connection.connect()
 
-    self.assertRaises(monitor.QmpCommandNotSupported,
-                      qmp_connection.Execute,
+      # Format the script
+      for request, expected_response in zip(self.REQUESTS,
+                                            self.EXPECTED_RESPONSES):
+        response = qmp_connection.Execute(request["execute"],
+                                          request["arguments"])
+        self.assertEqual(response, expected_response)
+        msg = hv_kvm.QmpMessage({"return": expected_response})
+        self.assertEqual(len(str(msg).splitlines()), 1,
+                         msg="Got multi-line message")
+
+      self.assertRaises(monitor.QmpCommandNotSupported,
+                        qmp_connection.Execute,
                       "unsupported-command")
+    finally:
+      qmp_stub.shutdown()
+      qmp_connection.close()
 
   def testQmpContextManager(self):
     # Set up the stub
@@ -238,12 +246,15 @@ class TestQmp(testutils.GanetiTestCase):
     qmp_stub = QmpStub(socket_file.name, self.SERVER_RESPONSES)
     qmp_stub.start()
 
-    # Test the context manager functionality
-    with hv_kvm.QmpConnection(socket_file.name) as qmp:
-      for request, expected_response in zip(self.REQUESTS,
-                                            self.EXPECTED_RESPONSES):
-        response = qmp.Execute(request["execute"], request["arguments"])
-        self.assertEqual(response, expected_response)
+    try:
+      # Test the context manager functionality
+      with hv_kvm.QmpConnection(socket_file.name) as qmp:
+        for request, expected_response in zip(self.REQUESTS,
+                                              self.EXPECTED_RESPONSES):
+          response = qmp.Execute(request["execute"], request["arguments"])
+          self.assertEqual(response, expected_response)
+    finally:
+      qmp_stub.shutdown()
 
 
 class TestConsole(unittest.TestCase):
@@ -588,15 +599,15 @@ class TestKvmRuntime(testutils.GanetiTestCase):
   def testDirectoriesCreated(self):
     hypervisor = hv_kvm.KVMHypervisor()
     self.mocks['ensure_dirs'].assert_called_with([
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/pid'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/uid'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/ctrl'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/conf'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/nic'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot-quarantine'), 0775),
-        (PostfixMatcher('/run/ganeti/kvm-hypervisor/keymap'), 0775)])
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/pid'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/uid'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/ctrl'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/conf'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/nic'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot-quarantine'), 0o775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/keymap'), 0o775)])
 
   def testStartInstance(self):
     hypervisor = hv_kvm.KVMHypervisor()
